@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 #
-# Copyright (c) 2013, 2014 Kevin Steves <kevin.steves@pobox.com>
+# Copyright (c) 2013-2014 Kevin Steves <kevin.steves@pobox.com>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -23,6 +23,7 @@ import signal
 import getopt
 import json
 import pprint
+import logging
 try:
     from urllib.parse import urlparse
 except ImportError:
@@ -31,6 +32,7 @@ except ImportError:
 libpath = os.path.dirname(os.path.abspath(__file__))
 sys.path[:0] = [os.path.join(libpath, os.pardir, 'lib')]
 import pan.wfapi
+import pan.config
 
 debug = 0
 
@@ -40,9 +42,24 @@ def main():
 #    set_encoding()
     options = parse_opts()
 
+    if options['debug']:
+        logger = logging.getLogger()
+        if options['debug'] == 3:
+            logger.setLevel(pan.wfapi.DEBUG3)
+        elif options['debug'] == 2:
+            logger.setLevel(pan.wfapi.DEBUG2)
+        elif options['debug'] == 1:
+            logger.setLevel(pan.wfapi.DEBUG1)
+
+#        log_format = '%(levelname)s %(name)s %(message)s'
+        log_format = '%(message)s'
+        handler = logging.StreamHandler()
+        formatter = logging.Formatter(log_format)
+        handler.setFormatter(formatter)
+        logger.addHandler(handler)
+
     try:
-        wfapi = pan.wfapi.PanWFapi(debug=options['debug'],
-                                   tag=options['tag'],
+        wfapi = pan.wfapi.PanWFapi(tag=options['tag'],
                                    api_key=options['api_key'],
                                    hostname=options['hostname'],
                                    timeout=options['timeout'],
@@ -302,12 +319,21 @@ def print_response(wfapi, options):
             print(wfapi.response_body)
 
         if options['print_python'] or options['print_json']:
+            if wfapi.xml_element_root is None:
+                return
+
+            elem = wfapi.xml_element_root
+            tags_forcelist = set(['entry'])
+
             try:
-                d = wfapi.xml_python()
-            except pan.wfapi.PanWFapiError as msg:
-                print('pan.wfapi.PanWFapi:', msg, file=sys.stderr)
+                conf = pan.config.PanConfig(config=elem,
+                                            tags_forcelist=tags_forcelist)
+            except pan.config.PanConfigError as msg:
+                print('pan.config.PanConfigError:', msg, file=sys.stderr)
                 sys.exit(1)
 
+            d = conf.python()
+
             if d:
                 if options['print_python']:
                     print('var1 =', pprint.pformat(d))
