@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 #
-# Copyright (c) 2013 Kevin Steves <kevin.steves@pobox.com>
+# Copyright (c) 2013-2014 Kevin Steves <kevin.steves@pobox.com>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -23,11 +23,13 @@ import getopt
 import re
 import json
 import pprint
+import logging
 
 libpath = os.path.dirname(os.path.abspath(__file__))
 sys.path[:0] = [os.path.join(libpath, os.pardir, 'lib')]
 import pan.xapi
 import pan.commit
+import pan.config
 
 debug = 0
 
@@ -36,9 +38,24 @@ def main():
     set_encoding()
     options = parse_opts()
 
+    if options['debug']:
+        logger = logging.getLogger()
+        if options['debug'] == 3:
+            logger.setLevel(pan.xapi.DEBUG3)
+        elif options['debug'] == 2:
+            logger.setLevel(pan.xapi.DEBUG2)
+        elif options['debug'] == 1:
+            logger.setLevel(pan.xapi.DEBUG1)
+
+#        log_format = '%(levelname)s %(name)s %(message)s'
+        log_format = '%(message)s'
+        handler = logging.StreamHandler()
+        formatter = logging.Formatter(log_format)
+        handler.setFormatter(formatter)
+        logger.addHandler(handler)
+
     try:
-        xapi = pan.xapi.PanXapi(debug=options['debug'],
-                                timeout=options['timeout'],
+        xapi = pan.xapi.PanXapi(timeout=options['timeout'],
                                 tag=options['tag'],
                                 use_http=options['use_http'],
                                 use_get=options['use_get'],
@@ -141,6 +158,13 @@ def main():
             print_status(xapi, action)
             print_response(xapi, options)
 
+        if options['override']:
+            action = 'override'
+            xapi.override(xpath=options['xpath'],
+                          element=options['element'])
+            print_status(xapi, action)
+            print_response(xapi, options)
+
         if options['export'] is not None:
             action = 'export'
             xapi.export(category=options['export'],
@@ -180,7 +204,7 @@ def main():
                 if options['cmd_xml']:
                     cmd = xapi.cmd_xml(cmd)
             else:
-                c = pan.commit.PanCommit(debug=options['debug'],
+                c = pan.commit.PanCommit(validate=options['validate'],
                                          force=options['force'],
                                          commit_all=options['commit_all'],
                                          merge_with_candidate=
@@ -239,6 +263,7 @@ def parse_opts():
         'set': False,
         'dynamic-update': False,
         'commit': False,
+        'validate': False,
         'force': False,
         'partial': [],
         'sync': False,
@@ -254,6 +279,7 @@ def parse_opts():
         'move': None,
         'rename': False,
         'clone': False,
+        'override': False,
         'api_username': None,
         'api_password': None,
         'hostname': None,
@@ -290,9 +316,9 @@ def parse_opts():
 
     short_options = 'de:gksS:U:C:A:o:l:h:P:K:xpjrXHGDt:T:'
     long_options = ['version', 'help',
-                    'ad-hoc=', 'modify', 'force', 'partial=', 'sync',
-                    'vsys=', 'src=', 'dst=', 'move=', 'rename',
-                    'clone', 'export=', 'log=', 'recursive',
+                    'ad-hoc=', 'modify', 'validate', 'force', 'partial=',
+                    'sync', 'vsys=', 'src=', 'dst=', 'move=', 'rename',
+                    'clone', 'override=', 'export=', 'log=', 'recursive',
                     'cafile=', 'capath=', 'ls', 'serial=',
                     'group=', 'merge', 'nlogs=', 'skip=', 'filter=',
                     'interval=', 'timeout=',
@@ -327,6 +353,8 @@ def parse_opts():
         elif opt == '-C':
             options['commit'] = True
             options['cmd'] = get_element(arg)
+        elif opt == '--validate':
+            options['validate'] = True
         elif opt == '--force':
             options['force'] = True
         elif opt == '--partial':
@@ -365,6 +393,9 @@ def parse_opts():
             options['rename'] = True
         elif opt == '--clone':
             options['clone'] = True
+        elif opt == '--override':
+            options['override'] = True
+            options['element'] = get_element(arg)
         elif opt == '-l':
             try:
                 (options['api_username'],
@@ -521,12 +552,24 @@ def print_response(xapi, options):
             print(s)
 
     if options['print_python'] or options['print_json']:
+        if options['print_result']:
+            if (xapi.element_result is None or
+                    not len(xapi.element_result)):
+                return
+            elem = list(xapi.element_result)[0]
+        else:
+            if xapi.element_root is None:
+                return
+            elem = xapi.element_root
+
         try:
-            d = xapi.xml_python(options['print_result'])
-        except pan.xapi.PanXapiError as msg:
-            print('pan.xapi.PanXapi:', msg, file=sys.stderr)
+            conf = pan.config.PanConfig(config=elem)
+        except pan.config.PanConfigError as msg:
+            print('pan.config.PanConfigError:', msg, file=sys.stderr)
             sys.exit(1)
 
+        d = conf.python()
+
         if d:
             if options['print_python']:
                 print('var1 =', pprint.pformat(d))
@@ -641,6 +684,7 @@ def usage():
     -S element            set XML element at xpath
     -U cmd                execute dynamic update command
     -C cmd                commit candidate configuration
+    --validate            validate candidate configuration
     --force               force commit when conflict
     --partial part        commit specified part
     --sync                synchronous commit
@@ -658,6 +702,7 @@ def usage():
     --move where          move after, before, bottom or top
     --rename              rename object at xpath to dst
     --clone               clone object at xpath, src xpath
+    --override element    override template object at xpath
     --vsys vsys           VSYS for dynamic update/partial commit/
                           operational command
     -l api_username:api_password
@@ -670,8 +715,8 @@ def usage():
     --nlogs num           retrieve num logs
     --skip num            skip num logs
     --filter filter       log selection filter
-    --interval            log/commit job query interval
-    --timeout             log/commit job query timeout
+    --interval seconds    log/commit job query interval
+    --timeout seconds     log/commit job query timeout
     -K api_key
     -x                    print XML response to stdout
     -p                    print XML response in Python to stdout
